@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks of the OX Software GmbH group of companies.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2011 Open-Xchange, Inc.
+ *     Copyright (C) 2016-2020 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,9 +49,9 @@
 
 package com.openexchange.spamhandler.spamexperts;
 
-import java.io.ByteArrayInputStream;
 import java.util.Properties;
 import javax.mail.MessagingException;
+import javax.mail.URLName;
 import javax.mail.internet.MimeMessage;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -59,12 +59,14 @@ import com.openexchange.exception.OXException;
 import com.openexchange.mail.MailField;
 import com.openexchange.mail.api.MailAccess;
 import com.openexchange.mail.dataobjects.MailMessage;
+import com.openexchange.mail.mime.utils.MimeMessageUtility;
 import com.openexchange.mail.service.MailService;
 import com.openexchange.session.Session;
 import com.openexchange.spamhandler.SpamHandler;
 import com.openexchange.spamhandler.spamexperts.exceptions.SpamExpertsExceptionCode;
 import com.openexchange.spamhandler.spamexperts.management.SpamExpertsConfig;
 import com.openexchange.spamhandler.spamexperts.osgi.SpamExpertsServiceRegistry;
+import com.openexchange.tools.ssl.TrustAllSSLSocketFactory;
 import com.sun.mail.imap.IMAPFolder;
 import com.sun.mail.imap.IMAPStore;
 
@@ -76,7 +78,7 @@ import com.sun.mail.imap.IMAPStore;
 public class SpamExpertsSpamHandler extends SpamHandler {
 
     private static final SpamExpertsSpamHandler instance = new SpamExpertsSpamHandler();
-    
+
     private static final Logger LOG = LoggerFactory.getLogger(SpamExpertsSpamHandler.class);
 
     /**
@@ -89,7 +91,7 @@ public class SpamExpertsSpamHandler extends SpamHandler {
     public static SpamExpertsSpamHandler getInstance() {
         return instance;
     }
-    
+
     /* (non-Javadoc)
      * @see com.openexchange.spamhandler.SpamHandler#getSpamHandlerName()
      */
@@ -98,43 +100,60 @@ public class SpamExpertsSpamHandler extends SpamHandler {
         return "SpamExperts";
     }
 
-    private void copyToSpamexpertsFolder(final String folder, final MailMessage []messages) throws OXException {
+    private void copyToSpamexpertsFolder(String folder, MailMessage[] messages) throws OXException {
+        final String socketFactoryClass = TrustAllSSLSocketFactory.class.getName();
+        final URLName imapUrl = SpamExpertsConfig.getInstance().getImapUrl();
         final Properties props = new Properties();
-        final javax.mail.Session imapSession = javax.mail.Session.getInstance(props);
-        final IMAPStore imapStore = new IMAPStore(imapSession, SpamExpertsConfig.getInstance().getImapUrl());
-        
+        if( "imaps".equals(imapUrl.getProtocol())) {
+            props.put("mail.imap.socketFactory.class", socketFactoryClass);
+        } else {
+            props.put("mail.imap.ssl.socketFactory.class", socketFactoryClass);
+            props.put("mail.imap.starttls.enable", "true");
+        }
+
+        props.put("mail.imap.socketFactory.port", imapUrl.getPort());
+        props.put("mail.imap.socketFactory.fallback", "false");
+
+        javax.mail.Session imapSession = javax.mail.Session.getInstance(props, null);
+
+        IMAPStore imapStore = null;
         try {
-            if( null == messages ) {
+            imapStore = (IMAPStore) imapSession.getStore("imap");
+            if (null == messages) {
                 throw SpamExpertsExceptionCode.UNABLE_TO_GET_MAILS.create();
             }
-            imapStore.connect(SpamExpertsConfig.getInstance().getImapUser(), SpamExpertsConfig.getInstance().getImappassword());
-            IMAPFolder sf = (IMAPFolder)imapStore.getFolder(folder);
-            if( ! sf.exists() ) {
+            imapStore.connect(imapUrl.getHost(), imapUrl.getPort(), SpamExpertsConfig.getInstance().getImapUser(), SpamExpertsConfig.getInstance().getImappassword());
+            IMAPFolder sf = (IMAPFolder) imapStore.getFolder(folder);
+            if (!sf.exists()) {
                 throw SpamExpertsExceptionCode.FOLDER_DOES_NOT_EXIST.create(folder);
             }
-            
-            MimeMessage []sfmesgs = new MimeMessage[messages.length];
-            
+
+            MimeMessage[] sfmesgs = new MimeMessage[messages.length];
+
             int i = 0;
-            for(final MailMessage mail : messages) {
-                sfmesgs[i++] = new MimeMessage(imapSession, new ByteArrayInputStream(mail.getSourceBytes()));
+            for (MailMessage mail : messages) {
+                if (null != mail) {
+                    sfmesgs[i++] = MimeMessageUtility.newMimeMessage(MimeMessageUtility.getStreamFromMailPart(mail), null);
+                }
             }
             sf.appendMessages(sfmesgs);
         } catch (MessagingException e) {
-            LOG.error(e.getMessage(),e);
+            LOG.error("", e);
             throw new OXException(e);
         } catch (OXException e) {
-            LOG.error(e.getMessage(),e);
+            LOG.error("", e);
             throw e;
         } finally {
-            if( null != imapStore && imapStore.isConnected() ) {
+            if ((null != imapStore) && imapStore.isConnected()) {
                 try {
                     imapStore.close();
-                } catch (MessagingException e) {}
+                } catch (Exception e) {
+                    // Ignore
+                }
             }
         }
     }
-    
+
     /* (non-Javadoc)
      * @see com.openexchange.spamhandler.SpamHandler#handleHam(int, java.lang.String, java.lang.String[], boolean, com.openexchange.session.Session)
      */
@@ -149,14 +168,14 @@ public class SpamExpertsSpamHandler extends SpamHandler {
             throw SpamExpertsExceptionCode.MAILSERVICE_MISSING.create();
         }
         MailAccess<?, ?> mailAccess = null;
-        
+
         try {
             mailAccess = mailService.getMailAccess(session, accountId);
             mailAccess.connect();
             final MailMessage[] mails = mailAccess.getMessageStorage().getMessages(spamFullName, mailIDs, new MailField[]{MailField.FULL});
 
             copyToSpamexpertsFolder(SpamExpertsConfig.getInstance().getTrainHamFolder(), mails);
-            
+
             if (move) {
                 /*
                  * Move to inbox
@@ -181,14 +200,14 @@ public class SpamExpertsSpamHandler extends SpamHandler {
             throw SpamExpertsExceptionCode.MAILSERVICE_MISSING.create();
         }
         MailAccess<?, ?> mailAccess = null;
-        
+
         try {
             mailAccess = mailService.getMailAccess(session, accountId);
             mailAccess.connect();
             final MailMessage[] mails = mailAccess.getMessageStorage().getMessages(fullName, mailIDs, new MailField[]{MailField.FULL});
 
             copyToSpamexpertsFolder(SpamExpertsConfig.getInstance().getTrainSpamFolder(), mails);
-            
+
             if (move) {
                 /*
                  * Move to spam folder (copied from spamassassin spamhandler)
